@@ -31,7 +31,7 @@
 #include <stdint.h>
 
 #define QALCULATE_MAJOR_VERSION (3)
-#define QALCULATE_MINOR_VERSION (3)
+#define QALCULATE_MINOR_VERSION (5)
 #define QALCULATE_MICRO_VERSION (0)
 
 /// \cond
@@ -319,6 +319,7 @@ typedef enum {
 #define BASE_PI			-7
 #define BASE_E			-8
 #define BASE_SQRT2		-9
+#define BASE_BIJECTIVE_26	-26
 
 #define EXP_BASE_3		-3
 #define EXP_PRECISION		-1
@@ -583,7 +584,8 @@ typedef enum {
 typedef enum {
 	COMPLEX_NUMBER_FORM_RECTANGULAR,
 	COMPLEX_NUMBER_FORM_EXPONENTIAL,
-	COMPLEX_NUMBER_FORM_POLAR
+	COMPLEX_NUMBER_FORM_POLAR,
+	COMPLEX_NUMBER_FORM_CIS
 } ComplexNumberForm;
 
 typedef enum {
